@@ -12,7 +12,7 @@ class DestructReferenceHandler < AbstractCommandHandler
         reference_cache = ReferencesCache.instance
         return reference_cache.delete_reference(command.payload[0])
       else
-        raise "Destruct Reference Handler parameters mismatch"
+        raise ArgumentError.new "Destruct Reference Handler parameters mismatch"
       end
     rescue Exception => e
       return e
