@@ -7,7 +7,7 @@ our @ISA = qw(Exporter);
 our @EXPORT = qw(get_command_type);
 
 my %command_type = (
-    'Response'                  => 0,
+    'Value'                  => 0,
     'LoadLibrary'               => 1,
     'InvokeStaticMethod'        => 2,
     'GetStaticField'            => 3,
