@@ -5,7 +5,7 @@ use Moose;
 use aliased 'Javonet::Sdk::Core::PerlCommand' => 'PerlCommand';
 use aliased 'Javonet::Sdk::Internal::InvocationContext' => 'InvocationContext';
 use aliased 'Javonet::Core::Handler::PerlHandler' => 'PerlHandler';
-use aliased 'Javonet::Core::Interpreter::Interpreter' => 'Interpreter';
+use aliased 'Javonet::Core::Interpreter::Interpreter' => 'Interpreter', qw(execute_);
 use aliased 'Javonet::Core::Exception::ExceptionThrower' => 'ExceptionThrower';
 
 extends 'Javonet::Sdk::Internal::Abstract::AbstractModuleContext',
@@ -22,7 +22,8 @@ sub new {
     my $self = {
         runtime_lib     => shift,
         connection_type => shift,
-        tcp_address     => shift
+        tcp_address     => shift,
+        response_command => 0,
     };
     bless $self, $class;
     return $self;
@@ -60,18 +61,17 @@ sub get_instance {
 
 
 sub execute {
+    my $self = $_[0];
     my $command = shift;
     my $connection_type = shift;
     my $tcp_address = shift;
-    my $response;
     if($command->{runtime} eq Javonet::Sdk::Core::RuntimeLib::get_runtime('Perl')) {
-        $response = $perl_handler->handle_command($command);
+        $self->{response_command} = $perl_handler->handle_command($command);
+    } else {
+        $self->{response_command} = Interpreter->execute_($command, $connection_type, $tcp_address);
     }
-    else{
-        $response = Interpreter->execute_($command, $connection_type, $tcp_address);
-    }
-    if ($response->{command_type} == Javonet::Sdk::Core::PerlCommandType::get_command_type('Exception')) {
-        ExceptionThrower->throwException($response)
+    if ($self->{response_command}->{command_type} == Javonet::Sdk::Core::PerlCommandType::get_command_type('Exception')) {
+        ExceptionThrower->throwException($self->{response_command})
     }
 }
 
@@ -132,17 +132,27 @@ sub cast {
 
 
 sub build_command {
-    my $self = shift;
-    my $command = shift;
-
-    if(!defined $self->{current_command}){
-        return $command;
+    my ($self, $command) = @_;
+    my $command_payload_length = @{$command->{payload}};
+    for (my $i = 0; $i < $command_payload_length; $i++) {
+        $command->{payload}[$i] = $self->encapsulate_payload_item($command->{payload}[$i]);
     }
-    else{
-        return $command->add_arg_to_payload_on_beginning($self->{current_command});
+    return $command->prepend_arg_to_payload($self->{current_command});
+}
+
+sub encapsulate_payload_item {
+    my ($self, $payload_item) = @_;
+    if ($payload_item->isa('Javonet::Sdk::Internal::InvocationContext')) {
+        # return Javonet::Sdk::Core::PerlCommand->new(
+        #     runtime      => $self->{runtime_name},
+        #     command_type => Javonet::Sdk::Core::PerlCommandType::get_command_type('Value'),
+        #     payload      => $payload_item->get_current_command(),
+        # );
+        return $payload_item->get_current_command();
+    } else {
+        return $payload_item;
     }
 }
 
-
 no Moose;
 1;
\ No newline at end of file
